@@ -40,13 +40,15 @@ def diff(a,b):
 	for x in a:
 		if b.get(x)!=a[x]: diff[x]=[a[x],b.get(x)]
 	return diff
-def notice(x,c='default'):request.urlopen(r'http://call.ff2.pw/?%s=%s&passwd=frank'%(c,urllib.parse.quote_plus(x)))
+def notice(x,c='default'):
+	try:request.urlopen(r'http://call.ff2.pw/?%s=%s&passwd=frank'%(c,urllib.parse.quote_plus(x)))
+	except:pass
 def rand_sleep(i=0.4,a=0.8):import random;time.sleep(random.randint(i*1000,a*1000)/1000);
 def rand(i=9,a=None):import random;return random.randint(int(i),int(a)) if a else random.randint(0,int(i-1));
 def md5(x):hl = hashlib.md5();hl.update(x.encode('utf-8') if isinstance(x,str) else x);return hl.hexdigest();
 def bin2hex(x):import binascii;return binascii.hexlify(x.encode('utf-8') if isinstance(x,str) else x);
 def base64_encode(x):import base64;return base64.b64encode(x) if isinstance(x,bytes) else base64.b64encode(x.encode('utf-8'))
-def base64_decode(x):import base64;return base64.b64decode(x)
+def base64_decode(x):import base64;return base64.b64decode(x+(b'===' if isinstance(x,bytes) else '==='))
 def log(*x,path=None):#已经带有时间
 	if path is None:
 		path='/tmp/frank.log' if os.path.exists('/tmp') else './FPC.log'
@@ -119,10 +121,18 @@ def end():
 	print(now('micro')-start_time)
 class mysql(object):
 	conn=None
-	def __init__(self, host=None,**args):
+	def __init__(self, host=None,port=3306,**args):
 		import pymysql
 		self.pymysql=pymysql
-		connect_config={'host':'127.0.0.1' if host is None else host,'user':'root','password':'','db':'localhost','charset':'utf8','cursorclass':pymysql.cursors.DictCursor,'autocommit':None}
+		if args.get('ssh_host'):
+			import sshtunnel
+			sshtunnel.DAEMON=True
+			sshtunnel.SSHTunnelForwarder.daemon_forward_servers=True
+			sshtunnel.SSHTunnelForwarder.daemon_transport=True
+			self.server = sshtunnel.SSHTunnelForwarder(ssh_address_or_host=args.pop('ssh_host'),ssh_username=args.pop('ssh_username'),ssh_password=args.pop('ssh_password'),remote_bind_address=('127.0.0.1' if host is None else host,port))
+			self.server.start()
+			port=self.server.local_bind_port
+		connect_config={'host':'127.0.0.1' if host is None else host,'port':port,'user':'root','password':'','db':'localhost','charset':'utf8','cursorclass':pymysql.cursors.DictCursor,'autocommit':None}
 		if 'table' in args:
 			self.table=args['table']
 			del args['table']
@@ -229,10 +239,10 @@ class redis(object):
 		import redis
 		if ssh_host:
 			import sshtunnel
-			sshtunnel.DAEMON=True 
+			sshtunnel.DAEMON=True
 			sshtunnel.SSHTunnelForwarder.daemon_forward_servers=True
 			sshtunnel.SSHTunnelForwarder.daemon_transport=True
-			self.server = sshtunnel.SSHTunnelForwarder(ssh_address_or_host=ssh_host,ssh_username=ssh_username,ssh_password=ssh_password,remote_bind_address=(host,6379))
+			self.server = sshtunnel.SSHTunnelForwarder(ssh_address_or_host=ssh_host,ssh_username=ssh_username,ssh_password=ssh_password,remote_bind_address=(host,port))
 			self.server.start()
 			port=self.server.local_bind_port
 		self.redis=redis.StrictRedis(host=host, port=port, db=db,decode_responses=True)
